# coding=utf-8
"""Top-level package for Django Model Validation Cleanup."""
from .errors import ExtensibleValidationError  # noqa F401
from .models import CleanMixin  # noqa F401

__author__ = """Janusz Skonieczny"""
__email__ = 'js+pypi@bravelabs.pl'
__version__ = '0.1.1'
